#ifndef CONCURRENTQUEUE_H
#define CONCURRENTQUEUE_H

#include <QSemaphore>

template <class T, const unsigned short S>
class ConcurrentQueue
{
public:
    ConcurrentQueue();
    void enqueue(const T &t);
    T dequeue();
    bool isEmpty() const;
private:
    T m_buffer[S];
    unsigned short m_next;
    unsigned short m_first;
    QSemaphore m_free;
    QSemaphore m_used;
};

template <class T, const unsigned short S>
ConcurrentQueue<T, S>::ConcurrentQueue() :
    m_next(0), m_first(0), m_free(S), m_used(0)
{
}

template <class T, const unsigned short S>
void ConcurrentQueue<T, S>::enqueue(const T &t)
{
    m_free.acquire();
    m_buffer[m_next] = t;
    m_next = (m_next + 1) % S;
    m_used.release();
}

template <class T, const unsigned short S>
T ConcurrentQueue<T, S>::dequeue()
{
    T temp;
    m_used.acquire();
    temp = m_buffer[m_first];
    m_first = (m_first + 1) % S;
    m_free.release();
    return temp;
}

template <class T, const unsigned short S>
bool ConcurrentQueue<T, S>::isEmpty() const
{
    return (m_first == m_next);
}

#endif // CONCURRENTQUEUE_H
