#ifndef VNC_H
#define VNC_H

#define DECODER_COUNT 16

#include <QQuickWindow>
#include <QObject>
#include <QQuickFramebufferObject>
#include <QThread>
#include <QOffscreenSurface>
#include <QMouseEvent>

#include "dispatcher.h"
#include "vncrenderer.h"
#include "uploader.h"

/*!
 * \brief The public interface of the VNC functionality
 */
class QVnc : public QQuickFramebufferObject
{
    Q_OBJECT
    Q_PROPERTY(QString host READ host WRITE setHost NOTIFY hostChanged)
    Q_PROPERTY(int port READ port WRITE setPort NOTIFY portChanged)

public:
    explicit QVnc(QQuickItem *parent = Q_NULLPTR);
    ~QVnc();

    Renderer *createRenderer() const;
    QString host() const;
    int port() const;
protected:
    void mousePressEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void keyReleaseEvent(QKeyEvent *event);
    void wheelEvent(QWheelEvent *event);
    void hoverMoveEvent(QHoverEvent *event);
private:
    Dispatcher *m_dispatcher;
    Uploader *m_uploader;
    QVector<JpegDecoder*> m_decoders;
    VncRenderer *m_renderer;
    QVector<QThread*> m_threads;

    QString m_host;
    int m_port;

    ulong m_lastMouseEvent;
signals:
    void open_connection(const QString host, const int port, const int width, const int height);
    void terminate_connection();

    void hostChanged(QString host);
    void portChanged(int port);

    void operate();
public slots:
    void open();
    void terminate();

    void setHost(QString host);
    void setPort(int port);

    void dispatch_error();
private slots:
    void winChanged(QQuickWindow *window);
};

#endif // VNC_H
