#ifndef UPLOADER_H
#define UPLOADER_H

#include <QObject>
#include <QQuickWindow>
#include <QDebug>
#include <QVector>
#include <QOffscreenSurface>
#include <QOpenGLContext>
#include <QOpenGLFunctions>
#include <QOpenGLBuffer>
#include <QOpenGLTexture>
#include <QOpenGLPixelTransferOptions>
#include <QOpenGLShaderProgram>
#include <QOpenGLFramebufferObject>
#include <QThreadPool>
#include <QtAlgorithms>
#include <QSharedPointer>
#include <QtMath>

class Bitmap {
public:
    QByteArray data;
    int x;
    int y;
    int width;
    int height;
};

class Uploader : public QObject
{
    Q_OBJECT
public:
    Uploader(QObject *parent = Q_NULLPTR);
    ~Uploader();

    QOpenGLBuffer vertices() const;
    QSharedPointer<QVector<GLuint>> textures() const;

    // VNC events
    void gotCopy(const int src_x, const int src_y, const int width, const int height, const int dest_x, const int dest_y);
    void gotFill(const int x, const int y, const int width, const int height, const float red, const float green, const float blue);
    int gotJpeg(const quint32 x, const quint32 y, const quint32 width, const quint32 height);
    void gotBitmap(const QByteArray &bitmap, const quint32 x, const quint32 y, const quint32 width, const quint32 height, const GLenum format, const GLenum type);
    void finishedUpdate();
private:
    QOffscreenSurface m_surface;
    QOpenGLContext m_context;
    QOpenGLFunctions *m_functions;

    QOpenGLBuffer m_vertices;
    unsigned int m_allocated;
    QSharedPointer<QVector<GLuint>> m_textures;

    QOpenGLShaderProgram m_program;

    GLuint m_FBOTexture;

    QVector<float> m_data;
    QVector<Bitmap *> m_bitmaps;
    int m_jpegs;
    bool m_finished;

    void swapBuffers();
signals:
    void uploadFinished();
public slots:
    void changeFBOTexture(const unsigned int FBOTexture, const QMatrix4x4 &ortho);
    void createContext(QOpenGLContext *context);
    void cleanup();

    void finishedJpeg(const unsigned char *image, const quint32 width, const quint32 height, int index);
};

#endif // UPLOADER_H
