#include "vncrenderer.h"
#include <cstring>

VncRenderer::VncRenderer(QQuickWindow *window,
                         QOpenGLBuffer vertices,
                         QSharedPointer<QVector<GLuint>> textures,
                         QObject *parent) :
    QObject(parent), m_window(window), m_vertices(vertices), m_textures(textures), m_rendered(false)
{
    connect(m_window, &QQuickWindow::frameSwapped, this, &VncRenderer::frameSwapped);
    m_program.addShaderFromSourceFile(QOpenGLShader::Vertex, "draw_shader.vsh");
    m_program.addShaderFromSourceFile(QOpenGLShader::Fragment, "draw_shader.fsh");
    m_program.bind();
    m_program.setUniformValue("texture", 0);
    m_program.release();

    m_functions = QOpenGLContext::currentContext()->functions();
    m_functions->glActiveTexture(GL_TEXTURE0);
}

void VncRenderer::render()
{
    /*static QTime frameTime;
    qDebug() << qRound(1000.0 / frameTime.elapsed()) << m_textures->length();
    frameTime.restart();*/

    m_program.bind();
    m_vertices.bind();

    m_program.setAttributeBuffer("position", GL_FLOAT, 0 * sizeof(float), 2, 7 * sizeof(float));
    m_program.setAttributeBuffer("color", GL_FLOAT, 2 * sizeof(float), 3, 7 * sizeof(float));
    m_program.setAttributeBuffer("texcoord", GL_FLOAT, 5 * sizeof(float), 2, 7 * sizeof(float));
    m_program.enableAttributeArray("position");
    m_program.enableAttributeArray("color");
    m_program.enableAttributeArray("texcoord");

    for(int i = 0; i < m_textures->length(); i++) {
        m_functions->glBindTexture(GL_TEXTURE_2D, m_textures->at(i));
        if(m_textures->at(i) != 0) {
            m_functions->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
            m_functions->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
            m_functions->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
            m_functions->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
        }
        m_functions->glDrawArrays(GL_TRIANGLE_STRIP, i * 4, 4);
    }
    m_window->resetOpenGLState();
    m_rendered = true;
}

QOpenGLFramebufferObject *VncRenderer::createFramebufferObject(const QSize &size)
{
    qDebug() << size;
    QMatrix4x4 ortho;
    ortho.ortho(QRect(QPoint(0, size.height()), QSize(size.width(), -1 * size.height())));
    m_program.bind();
    m_program.setUniformValue("ortho", ortho);
    m_program.release();

    QOpenGLFramebufferObjectFormat format;
    format.setTextureTarget(GL_TEXTURE_2D);
#ifdef GL_RGBA8
    format.setInternalTextureFormat(GL_RGBA8);
#else
    #ifdef GL_RGBA8_OES
        format.setInternalTextureFormat(GL_RGBA8_OES);
    #else
        format.setInternalTextureFormat(GL_RGBA);
    #endif
#endif

    QOpenGLFramebufferObject *framebufferObject = new QOpenGLFramebufferObject(size, format);
    emit FBOTextureChanged(framebufferObject->texture(), ortho);
    return framebufferObject;
}

void VncRenderer::setColorMax(const QVector3D &colorMax)
{
    if(m_program.bind()) {
        m_program.setUniformValue("colormax", colorMax);
        m_program.release();
    }
}

void VncRenderer::frameSwapped()
{
    if(m_rendered) {
        m_rendered = false;
        emit finishedRendering();
    }
}
