#ifndef VNCRENDERER_H
#define VNCRENDERER_H

#include <QQuickWindow>
#include <QQuickFramebufferObject>
#include <QOpenGLFramebufferObject>
#include <QOpenGLContext>
#include <QOpenGLFunctions>
#include <QMatrix4x4>
#include <QOpenGLShaderProgram>
#include <QOpenGLBuffer>
#include <QSharedPointer>
#include <QTime>

class VncRenderer : public QObject, public QQuickFramebufferObject::Renderer
{
    Q_OBJECT
public:
    VncRenderer(QQuickWindow *window,
                QOpenGLBuffer vertices,
                QSharedPointer<QVector<GLuint>> textures,
                QObject *parent = 0);
protected:
    void render() override;
    QOpenGLFramebufferObject *createFramebufferObject(const QSize &size) override;
private:
    QQuickWindow *m_window;
    QOpenGLBuffer m_vertices;
    QSharedPointer<QVector<GLuint>> m_textures;
    QVector3D m_colorMax;
    QOpenGLShaderProgram m_program;
    QOpenGLFunctions *m_functions;
    bool m_rendered;
public slots:
    void setColorMax(const QVector3D &colorMax);
private slots:
    void frameSwapped();
signals:
    void FBOTextureChanged(const unsigned int FBOTexture, const QMatrix4x4 &ortho);
    void finishedRendering();
};

#endif // VNCRENDERER_H
